import os
import setuptools

required = [
    "pandas", 
    "webdriver_manager == 3.8.3", 
    "selenium >= 4.1.0"
    ]


setuptools.setup(

    
    # license="MIT",
    
    # #long_description=long_description,
    
    # #long_description_content_type="text/markdown",
    
    # url="https://github.com/voulkon/scrape_reviews",
    
    install_requires=required,
      
    keywords = ["scrape","reviews","maps","google","gmaps","hotels","restaurants","hotel_reviews", "restaurant_reviews"],
    
    #project_urls = {"Source Code":"https://github.com/voulkon/scrape_reviews"},
    
    # packages=setuptools.find_packages(),
    
    # classifiers=[
    #     "Programming Language :: Python :: 3",
    #     "License :: OSI Approved :: MIT License",
    #     "Operating System :: OS Independent",
    # ],
)


# -*- coding: utf-8 -*-

from scrape_reviews.categories_of_interest import GoogleMapsCategories
import os.path as path

FOUND_IN_ATTRIBUTES = {

    'reviewer':'aria-label',
    'data-review-id': 'data-review-id'

}

FOUND_IN_CLASSES = {

    'text_of_review' :'wiI7pd',
    'date_created' : 'xRkPPb',
    'reviewer': 'd4r55',
    'reviewers_no_of_reviews':'RfnDt',
    'rating':'fzvQIb'

}

INFO_DICTIONARY = {

        'Name' : 'qBF1Pd',
        'Overall_Rating':'MW4etd',
        'Number_of_Ratings':'UY7F9'

    }


#Depends on the download speed of the server
GENERAL_WAIT_TIME = 1.5
CLASS_OF_REVIEW_CONTAINER = "jftiEf"
CLASS_OF_REVIEWS_BUTTON = "DkEaL" #"hh2c6.G7m0Af" #""
REVIEWS_WINDOWS_CLASS = "#QA0Szd > div > div > div.w6VYqd > div.bJzME.Hu9e2e.tTVLSc > div > div.e07Vkf.kA9KIf > div > div > div.m6QErb.DxyBCb.kA9KIf.dS8AEf"
CLASS_OF_CATEGORIES_OF_INTEREST_BUTTONS = "KNfEk.Rqu0ae"

ENTITIES_BASE_XPATH_INIT = '//*[@id="QA0Szd"]/div/div/div[1]/div[2]/div/div[1]/div/div/div[{to_be_confirmed}]/div[1]'

CLASS_OF_STARS = 'kvMYJc'

#The Button to Click in order to Avoid Cookies Screen and procceed with the actul website
COOKIES_XPATH = '//*[@id="yDmH0d"]/c-wiz/div/div/div/div[2]/div[1]/div[3]/div[1]/div[1]/form[1]/div/div/button'

PARENT_XPATH_OF_CATEGORIES = '//*[@id="assistive-chips"]/div/div/div/div[1]/div/div/div/div/div[5]/div[2]'

XPATH_OF_EACH_CATEGORY = {
    '//*[@id="QA0Szd"]/div/div/div[1]/div[2]/div/div[1]/div/div/div[{to_be_confirmed}]/div[1]' : GoogleMapsCategories.HOTELS,
    '//*[@id="QA0Szd"]/div/div/div[1]/div[2]/div/div[1]/div/div/div[{to_be_confirmed}]' : GoogleMapsCategories.RESTAURANTS
    }


# #For future reference - not used for now
# with open(f"{path.dirname(path.abspath(__file__)) }/greek_stopwords.txt","r", encoding = "utf-8") as f:
#     greek_stopwords = f.readlines()


import sqlite3
import pandas as pd
import logging

class DbConnector:
    
    def __init__(self,db_name):
        
        self.db_name = db_name #Store as a property in case it's needed
        self.connection = sqlite3.connect(self.db_name )
        logging.info(f"Connected to: {self.db_name}" )
        self.cur = self.connection.cursor()
        self.attempt_to_create_tables()

    def create_entity_info_table(self):

        self.cur.execute("""

        CREATE TABLE IF NOT EXISTS entity_info (
            Name TEXT NOT NULL PRIMARY KEY,
            Overall_Rating TEXT(3),
            Number_of_Ratings TEXT(10),
            'fetched_looking_at_place' TEXT(50),
            fetched_at TEXT(19)
            );
            """
            )

    def create_reviews_table(self):

        self.cur.execute("""

        CREATE TABLE IF NOT EXISTS reviews (
            reviewers_no_of_reviews TEXT(30),
            reviewer TEXT(80),
            rating TEXT(3),
            text_of_review TEXT,
            regards_entity TEXT(80),
            date_created TEXT(30),
            fetched_at TEXT(19),
            'data-review-id' TEXT(36) NOT NULL PRIMARY KEY
            );
            """
            )

    def attempt_to_create_tables(self):
        self.create_entity_info_table()
        self.create_reviews_table()

    def insert_dataframe_in_table(
        self,
        dataframe_to_write_in_db:pd.DataFrame,
        table_name:str
        ):
        dataframe_to_write_in_db.to_sql(name = table_name, con = self.connection, if_exists = 'append',index = False)

    def read_a_table(
        self,
        table_wanted
    ):
        query_to_fetch_it_all = """SELECT * FROM {t}""".format(t = table_wanted)
        self.cur.execute(query_to_fetch_it_all)
        rows = self.cur.fetchall()
        return rows

    def search_reviews(
        self,
        search_keyword
    ):
        keyword_search_query = """ 
        SELECT * 
        FROM reviews 
        WHERE text_of_review LIKE '%{keyword}%'""".format(keyword = search_keyword)

        return self.cur.execute(keyword_search_query).fetchall()
    
    def insert_values_in_table(
        self,
        table_wanted:str,
        items:list        
     ):
        items_part = ",".join(["?" for i,item in enumerate(items)])
        insert_values_query = """
        INSERT OR
        IGNORE INTO
        {table_wanted}
        VALUES({items_part})
        """
        
        insert_values_query_formatted = insert_values_query.format(table_wanted = table_wanted,items_part = items_part )
        
        self.cur.execute(insert_values_query_formatted,items)
        
        self.connection.commit() 
    
    def insert_values_in_table_by_colname(
        self,
        table_wanted:str,
        named_items:dict        
     ):

        columns_part =  ",".join(["'"+ k + "'" for k in named_items.keys()])

        values_part = ",".join([ "'" + v.replace("'", " " ) + "'" for v in named_items.values()])
        
        insert_named_values_query = """
        INSERT OR
        IGNORE INTO
        {table_wanted} ({columns_part})
        VALUES ({values_part})
        """
        
        insert_named_values_query_formatted = insert_named_values_query.format(
            table_wanted = table_wanted,
            values_part = values_part,
            columns_part = columns_part 
            )
        
        self.cur.execute(insert_named_values_query_formatted)
        
        self.connection.commit() 

    def perform_a_query(self,query):
        return self.cur.execute(query).fetchall()



# -*- coding: utf-8 -*-
"""

The gmaps module contains functions that attempt to scrape
all reviews for a certain place or search term.

"""

#

__docformat__ = "restructuredtext"

#Check that hard dependencies exist
# Let users know if they're missing
_hard_dependencies = ("pandas", "selenium", "pytz", "webdriver_manager")
_missing_dependencies = []

for _dependency in _hard_dependencies:
    
    try:
        __import__(_dependency)
    except ImportError as _e:
        _missing_dependencies.append(f"{_dependency}: {_e}")

if _missing_dependencies:
    raise ImportError(
        "Unable to import required dependencies:\n" + "\n".join(_missing_dependencies)
    )
del _hard_dependencies, _dependency, _missing_dependencies
